@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2014 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2014-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,16 +19,12 @@
 
 #include <gpaste-applet.h>
 
-#include <glib/gi18n.h>
-
-#include <stdlib.h>
-
 gint
 main (gint argc, gchar *argv[])
 {
     G_PASTE_INIT_APPLICATION ("Applet");
 
-    G_GNUC_UNUSED G_PASTE_CLEANUP_UNREF GPasteApplet *applet = g_paste_applet_new_status_icon (app);
+    G_GNUC_UNUSED g_autoptr (GPasteApplet) applet = g_paste_applet_new_status_icon (app);
 
     return g_application_run (gapp, argc, argv);
 }
