@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2014 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2014-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -29,10 +29,14 @@ const DeleteButton = Me.imports.deleteButton;
 
 const GPasteDeleteItemPart = new Lang.Class({
     Name: 'GPasteDeleteItemPart',
-    Extends: St.Bin,
 
     _init: function(client, index) {
-        this.parent({ x_align: St.Align.END });
-        this.child = new DeleteButton.GPasteDeleteButton(client, index);
+        this.actor = new St.Bin({ x_align: St.Align.END });
+        this._deleteButton = new DeleteButton.GPasteDeleteButton(client, index);
+        this.actor.child = this._deleteButton.actor;
+    },
+
+    setIndex: function(index) {
+        this._deleteButton.setIndex(index);
     }
 });
