@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2011-2014 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,15 +17,19 @@
  *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gpaste-clipboard-private.h"
-
+#include <gpaste-clipboard.h>
 #include <gpaste-image-item.h>
-#include <gpaste-password-item.h>
 #include <gpaste-uris-item.h>
+#include <gpaste-util.h>
 
 #include <string.h>
 
-struct _GPasteClipboardPrivate
+struct _GPasteClipboard
+{
+    GObject parent_instance;
+};
+
+typedef struct
 {
     GdkAtom         target;
     GtkClipboard   *real;
@@ -34,7 +38,7 @@ struct _GPasteClipboardPrivate
     gchar          *image_checksum;
 
     gulong          owner_change_signal;
-};
+} GPasteClipboardPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GPasteClipboard, g_paste_clipboard, G_TYPE_OBJECT)
 
@@ -45,7 +49,75 @@ enum
     LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
+static guint64 signals[LAST_SIGNAL] = { 0 };
+
+static void
+g_paste_clipboard_bootstrap_finish (GPasteClipboard *self,
+                                    GPasteHistory   *history)
+{
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+
+    if (!priv->text && !priv->image_checksum)
+    {
+        const GSList *h = g_paste_history_get_history (history);
+        if (h)
+            g_paste_clipboard_select_item (self, h->data);
+    }
+}
+
+static void
+g_paste_clipboard_bootstrap_finish_text (GPasteClipboard *self,
+                                         const gchar     *text G_GNUC_UNUSED,
+                                         gpointer         user_data)
+{
+    g_paste_clipboard_bootstrap_finish (self, user_data);
+}
+
+static void
+g_paste_clipboard_bootstrap_finish_image (GPasteClipboard *self,
+                                          GdkPixbuf       *image G_GNUC_UNUSED,
+                                          gpointer         user_data)
+{
+    g_paste_clipboard_bootstrap_finish (self, user_data);
+}
+
+/**
+ * g_paste_clipboard_bootstrap:
+ * @self: a #GPasteClipboard instance
+ * @history: a #GPasteHistory instance
+ *
+ * Bootstrap a #GPasteClipboard with an initial value
+ *
+ * Returns:
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_bootstrap (GPasteClipboard *self,
+                             GPasteHistory   *history)
+{
+    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (G_PASTE_IS_HISTORY (history));
+
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    GtkClipboard *real = priv->real;
+
+    if (gtk_clipboard_wait_is_uris_available (real) ||
+        gtk_clipboard_wait_is_text_available (real))
+    {
+        g_paste_clipboard_set_text (self,
+                                    g_paste_clipboard_bootstrap_finish_text,
+                                    history);
+    }
+    else if (g_paste_settings_get_images_support (priv->settings) && gtk_clipboard_wait_is_image_available (real))
+    {
+        g_paste_clipboard_set_image (self,
+                                     g_paste_clipboard_bootstrap_finish_image,
+                                     history);
+    }
+    else
+    {
+        g_paste_clipboard_ensure_not_empty (self, history);
+    }
+}
 
 /**
  * g_paste_clipboard_get_target:
@@ -60,7 +132,7 @@ g_paste_clipboard_get_target (const GPasteClipboard *self)
 {
     g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private ((GPasteClipboard *) self);
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
     return priv->target;
 }
@@ -78,7 +150,7 @@ g_paste_clipboard_get_real (const GPasteClipboard *self)
 {
     g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private ((GPasteClipboard *) self);
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
     return priv->real;
 }
@@ -96,7 +168,7 @@ g_paste_clipboard_get_text (const GPasteClipboard *self)
 {
     g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private ((GPasteClipboard *) self);
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
     return priv->text;
 }
@@ -123,7 +195,7 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
                                  const gchar  *text,
                                  gpointer      user_data)
 {
-    G_PASTE_CLEANUP_FREE GPasteClipboardTextCallbackData *data = user_data;
+    g_autofree GPasteClipboardTextCallbackData *data = user_data;
     GPasteClipboard *self = data->self;
 
     if (!text)
@@ -135,10 +207,10 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
     GPasteSettings *settings = priv->settings;
-    G_PASTE_CLEANUP_FREE gchar *stripped = g_strstrip (g_strdup (text));
+    g_autofree gchar *stripped = g_strstrip (g_strdup (text));
     gboolean trim_items = g_paste_settings_get_trim_items (settings);
     const gchar *to_add = trim_items ? stripped : text;
-    gsize length = strlen (to_add);
+    guint64 length = strlen (to_add);
 
     if (length < g_paste_settings_get_min_text_item_size (settings) ||
         length > g_paste_settings_get_max_text_item_size (settings) ||
@@ -226,7 +298,7 @@ g_paste_clipboard_select_text (GPasteClipboard *self,
 static void
 g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
                                       GtkSelectionData *selection_data,
-                                      guint             info G_GNUC_UNUSED,
+                                      guint32           info      G_GNUC_UNUSED,
                                       gpointer          user_data_or_owner)
 {
     g_return_if_fail (G_PASTE_IS_ITEM (user_data_or_owner));
@@ -255,20 +327,16 @@ g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
         /* The content is requested as special gnome-copied-files by nautilus */
         else
         {
-            G_PASTE_CLEANUP_STRING_FREE GString *copy_string = g_string_new ("copy");
-            guint length = g_strv_length ((GStrv) uris);
+            g_autoptr (GString) copy_string = g_string_new ("copy");
+            guint64 length = g_strv_length ((GStrv) uris);
 
-            for (guint i = 0; i < length; ++i)
-            {
-                g_string_append (g_string_append (copy_string,
-                                                  "\n"),
-                                 uris[i]);
-            }
+            for (guint64 i = 0; i < length; ++i)
+                g_string_append_printf (copy_string, "\n%s", uris[i]);
 
             gchar *str = copy_string->str;
             length = copy_string->len + 1;
-            G_PASTE_CLEANUP_FREE guchar *copy_files_data = g_new (guchar, length);
-            for (guint i = 0; i < length; ++i)
+            g_autofree guchar *copy_files_data = g_new (guchar, length);
+            for (guint64 i = 0; i < length; ++i)
                 copy_files_data[i] = (guchar) str[i];
             gtk_selection_data_set (selection_data, g_paste_clipboard_copy_files_target, 8, copy_files_data, length);
         }
@@ -287,7 +355,7 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
                                        GPasteUrisItem         *item)
 {
     GtkClipboard *real = priv->real;
-    G_PASTE_CLEANUP_TARGETS_UNREF GtkTargetList *target_list = gtk_target_list_new (NULL, 0);
+    g_autoptr (GtkTargetList) target_list = gtk_target_list_new (NULL, 0);
 
     g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (G_PASTE_ITEM (item)));
 
@@ -295,7 +363,7 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
     gtk_target_list_add_uri_targets (target_list, 0);
     gtk_target_list_add (target_list, g_paste_clipboard_copy_files_target, 0, 0);
 
-    gint n_targets;
+    gint32 n_targets;
     GtkTargetEntry *targets = gtk_target_table_new_from_list (target_list, &n_targets);
     gtk_clipboard_set_with_owner (real,
                                   targets,
@@ -308,6 +376,27 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
     gtk_target_table_free (targets, n_targets);
 }
 
+/**
+ * g_paste_clipboard_clear:
+ * @self: a #GPasteClipboard instance
+ *
+ * Clears the content of the clipboard
+ *
+ * Returns:
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_clear (GPasteClipboard *self)
+{
+    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+
+    g_clear_pointer (&priv->text, g_free);
+    g_clear_pointer (&priv->image_checksum, g_free);
+
+    gtk_clipboard_clear (priv->real);
+}
+
 /**
  * g_paste_clipboard_get_image_checksum:
  * @self: a #GPasteClipboard instance
@@ -321,7 +410,7 @@ g_paste_clipboard_get_image_checksum (const GPasteClipboard *self)
 {
     g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private ((GPasteClipboard *) self);
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
     return priv->image_checksum;
 }
@@ -342,13 +431,12 @@ g_paste_clipboard_private_select_image (GPasteClipboardPrivate *priv,
                                         GdkPixbuf              *image,
                                         const gchar            *checksum)
 {
-    g_return_if_fail (image);
+    g_return_if_fail (GDK_IS_PIXBUF (image));
 
     GtkClipboard *real = priv->real;
 
     g_paste_clipboard_private_set_image_checksum (priv, checksum);
     gtk_clipboard_set_image (real, image);
-    gtk_clipboard_store (real);
 }
 
 typedef struct {
@@ -362,7 +450,7 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
                                   GdkPixbuf    *image,
                                   gpointer      user_data)
 {
-    G_PASTE_CLEANUP_FREE GPasteClipboardImageCallbackData *data = user_data;
+    g_autofree GPasteClipboardImageCallbackData *data = user_data;
     GPasteClipboard *self = data->self;
 
     if (!image)
@@ -374,7 +462,7 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    G_PASTE_CLEANUP_FREE gchar *checksum = g_compute_checksum_for_data (G_CHECKSUM_SHA256,
+    g_autofree gchar *checksum = g_compute_checksum_for_data (G_CHECKSUM_SHA256,
                                                                         (guchar *) gdk_pixbuf_get_pixels (image),
                                                                         -1);
 
@@ -383,16 +471,14 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
         g_paste_clipboard_private_select_image (priv,
                                                 image,
                                                 checksum);
-
-        if (data->callback)
-            data->callback (self, image, data->user_data);
     }
     else
     {
-        if (data->callback)
-            data->callback (self, NULL, data->user_data);
-        g_object_unref (image);
+        image = NULL;
     }
+
+    if (data->callback)
+        data->callback (self, image, data->user_data);
 }
 
 /**
@@ -470,6 +556,28 @@ g_paste_clipboard_select_item (GPasteClipboard  *self,
     }
 }
 
+/**
+ * g_paste_clipboard_ensure_not_empty:
+ * @self: a #GPasteClipboard instance
+ * @history: a #GPasteHistory instance
+ *
+ * Ensure the clipboard has some contents (as long as the history's not empty)
+ *
+ * Returns:
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_ensure_not_empty (GPasteClipboard     *self,
+                                    const GPasteHistory *history)
+{
+    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (G_PASTE_IS_HISTORY (history));
+
+    const GSList *hist = g_paste_history_get_history (history);
+
+    if (hist)
+        g_paste_clipboard_select_item (self, hist->data);
+}
+
 static void
 g_paste_clipboard_owner_change (GtkClipboard        *clipboard G_GNUC_UNUSED,
                                 GdkEventOwnerChange *event,
@@ -484,6 +592,49 @@ g_paste_clipboard_owner_change (GtkClipboard        *clipboard G_GNUC_UNUSED,
                    NULL);
 }
 
+static void
+g_paste_clipboard_fake_event_finish_text (GtkClipboard *clipboard G_GNUC_UNUSED,
+                                          const gchar  *text,
+                                          gpointer      user_data)
+{
+    GPasteClipboard *self = user_data;
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+
+    if (g_strcmp0 (text, priv->text))
+        g_paste_clipboard_owner_change (NULL, NULL, self);
+}
+
+static void
+g_paste_clipboard_fake_event_finish_image (GtkClipboard *clipboard G_GNUC_UNUSED,
+                                           GdkPixbuf    *image,
+                                           gpointer      user_data)
+{
+    GPasteClipboard *self = user_data;
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    g_autofree gchar *checksum = g_paste_util_compute_checksum (image);
+
+    if (g_strcmp0 (checksum, priv->image_checksum))
+        g_paste_clipboard_owner_change (NULL, NULL, self);
+
+    g_object_unref (image);
+}
+
+static gboolean
+g_paste_clipboard_fake_event (gpointer user_data)
+{
+    GPasteClipboard *self = user_data;
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+
+    if (priv->text)
+        gtk_clipboard_request_text (priv->real, g_paste_clipboard_fake_event_finish_text, self);
+    else if (priv->image_checksum)
+        gtk_clipboard_request_image (priv->real, g_paste_clipboard_fake_event_finish_image, self);
+    else
+        g_paste_clipboard_owner_change (NULL, NULL, self);
+
+    return G_SOURCE_CONTINUE;
+}
+
 static void
 g_paste_clipboard_dispose (GObject *object)
 {
@@ -517,6 +668,15 @@ g_paste_clipboard_class_init (GPasteClipboardClass *klass)
     object_class->dispose = g_paste_clipboard_dispose;
     object_class->finalize = g_paste_clipboard_finalize;
 
+    /**
+     * GPasteClipboard::owner-change:
+     * @clipboard: the object on which the signal was emitted
+     * @event: (type Gdk.EventOwnerChange): the @GdkEventOwnerChange event
+     *
+     * The "owner-change" signal is emitted when GPaste receives an
+     * event that indicates that the ownership of the selection
+     * associated with @clipboard has changed.
+     */
     signals[OWNER_CHANGE] = g_signal_new ("owner-change",
                                           G_PASTE_TYPE_CLIPBOARD,
                                           G_SIGNAL_RUN_FIRST,
@@ -554,9 +714,6 @@ g_paste_clipboard_new (GdkAtom         target,
 {
     g_return_val_if_fail (G_PASTE_IS_SETTINGS (settings), NULL);
 
-    if (!gdk_display_request_selection_notification (gdk_display_get_default (), target))
-        g_critical ("Clipboard notifications not supported, GPaste won't work (XFixes not found).");
-
     GPasteClipboard *self = g_object_new (G_PASTE_TYPE_CLIPBOARD, NULL);
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
@@ -570,5 +727,11 @@ g_paste_clipboard_new (GdkAtom         target,
                                                   G_CALLBACK (g_paste_clipboard_owner_change),
                                                   self);
 
+    if (!gdk_display_request_selection_notification (gdk_display_get_default (), target))
+    {
+        g_warning ("Selection notification not supported, using active poll");
+        g_source_set_name_by_id (g_timeout_add_seconds (1, g_paste_clipboard_fake_event, self), "[GPaste] clipboard fake events");
+    }
+
     return self;
 }
