@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2011-2013 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -28,20 +28,14 @@
 
 G_BEGIN_DECLS
 
-/* GPaste TextItem */
+#define G_PASTE_TYPE_TEXT_ITEM (g_paste_text_item_get_type ())
 
-#define G_PASTE_TYPE_TEXT_ITEM                (g_paste_text_item_get_type ())
-#define G_PASTE_TEXT_ITEM(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_PASTE_TYPE_TEXT_ITEM, GPasteTextItem))
-#define G_PASTE_IS_TEXT_ITEM(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_PASTE_TYPE_TEXT_ITEM))
-#define G_PASTE_TEXT_ITEM_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), G_PASTE_TYPE_TEXT_ITEM, GPasteTextItemClass))
-#define G_PASTE_IS_TEXT_ITEM_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE ((klass), G_PASTE_TYPE_TEXT_ITEM))
-#define G_PASTE_TEXT_ITEM_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), G_PASTE_TYPE_TEXT_ITEM, GPasteTextItemClass))
+G_PASTE_DERIVABLE_TYPE (TextItem, text_item, TEXT_ITEM, GPasteItem)
 
-typedef struct _GPasteTextItem GPasteTextItem;
-typedef struct _GPasteTextItemClass GPasteTextItemClass;
-
-G_PASTE_VISIBLE
-GType g_paste_text_item_get_type (void);
+struct _GPasteTextItemClass
+{
+    GPasteItemClass parent_class;
+};
 
 GPasteItem *g_paste_text_item_new (const gchar *text);
 
