@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2013-2014 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2013-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,76 +17,25 @@
  *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#if !defined (__G_PASTE_H_INSIDE__) && !defined (G_PASTE_COMPILATION)
+#error "Only <gpaste.h> can be included directly."
+#endif
+
 #ifndef __G_PASTE_GDBUS_MACROS_H__
 #define __G_PASTE_GDBUS_MACROS_H__
 
+#include <gpaste-gdbus-defines.h>
+#include <gpaste-util.h>
+
+#include <gio/gio.h>
+
 G_BEGIN_DECLS
 
-/*********************/
-/* Custom Data Types */
-/*********************/
-
-#ifdef __G_PASTE_NEEDS_BS__
-typedef struct
-{
-    gboolean b;
-    gchar   *s;
-} GPasteDBusBSResult;
-#endif /* __G_PASTE_NEEDS_BS__ */
-
-/*********************/
-/* Custom Extractors */
-/*********************/
-
-#ifdef __G_PASTE_NEEDS_BS__
-static GPasteDBusBSResult
-g_paste_dbus_get_bs_result (GVariant *variant)
-{
-    GVariantIter iter;
-    g_variant_iter_init (&iter, variant);
-
-    G_PASTE_CLEANUP_VARIANT_UNREF GVariant *b = g_variant_iter_next_value (&iter);
-    G_PASTE_CLEANUP_VARIANT_UNREF GVariant *s = g_variant_iter_next_value (&iter);
-
-    GPasteDBusBSResult r = {
-        .b = g_variant_get_boolean (b),
-        .s = g_variant_dup_string (s, NULL /* length */)
-    };
-
-    return r;
-}
-#endif /* __G_PASTE_NEEDS_BS__ */
-
-#ifdef __G_PASTE_NEEDS_AU__
-static guint32 *
-g_paste_dbus_get_au_result (GVariant *variant)
-{
-    GVariantIter iter;
-    g_variant_iter_init (&iter, variant);
-    GVariant *loop;
-    guint alloc = 4;
-    guint32 *res = g_new (guint32, alloc);
-
-    for (guint32 i = 0; (loop = g_variant_iter_next_value (&iter)); ++i)
-    {
-        if (i > alloc)
-        {
-            alloc *=2;
-            res = g_renew (guint32, res, alloc);
-        }
-        G_PASTE_CLEANUP_VARIANT_UNREF GVariant *v = loop;
-        res[i] = g_variant_get_uint32 (v);
-    }
-
-    return res;
-}
-#endif /* __G_PASTE_NEEDS_AU__ */
-
 /***************/
 /* Constructor */
 /***************/
 
-#define CUSTOM_PROXY_NEW_ASYNC(TYPE, BUS_ID)                                           \
+#define CUSTOM_PROXY_NEW_ASYNC(TYPE, BUS_ID, BUS_NAME)                                 \
     g_async_initable_new_async (G_PASTE_TYPE_##TYPE,                                   \
                                 G_PRIORITY_DEFAULT,                                    \
                                 NULL, /* cancellable */                                \
@@ -94,26 +43,28 @@ g_paste_dbus_get_au_result (GVariant *variant)
                                 user_data,                                             \
                                 "g-bus-type",       G_BUS_TYPE_SESSION,                \
                                 "g-flags",          G_DBUS_PROXY_FLAGS_NONE,           \
-                                "g-name",           G_PASTE_##BUS_ID##_BUS_NAME,       \
+                                "g-name",           BUS_NAME,                          \
                                 "g-object-path",    G_PASTE_##BUS_ID##_OBJECT_PATH,    \
                                 "g-interface-name", G_PASTE_##BUS_ID##_INTERFACE_NAME, \
                                 NULL)
 
-#define CUSTOM_PROXY_NEW_FINISH(TYPE)                                                  \
-    G_PASTE_CLEANUP_UNREF GObject *source = g_async_result_get_source_object (result); \
-    g_assert (source);                                                                 \
-    GObject *self = g_async_initable_new_finish (G_ASYNC_INITABLE (source),            \
-                                                 result,                               \
-                                                 error);                               \
+#define CUSTOM_PROXY_NEW_FINISH(TYPE)                                       \
+    g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);                \
+    g_return_val_if_fail (!error || !(*error), NULL);                       \
+    g_autoptr (GObject) source = g_async_result_get_source_object (result); \
+    g_assert (source);                                                      \
+    GObject *self = g_async_initable_new_finish (G_ASYNC_INITABLE (source), \
+                                                 result,                    \
+                                                 error);                    \
     return (self) ? G_PASTE_##TYPE (self) : NULL;
 
-#define CUSTOM_PROXY_NEW(TYPE, BUS_ID)                                                       \
+#define CUSTOM_PROXY_NEW(TYPE, BUS_ID, BUS_NAME)                                             \
     GInitable *self = g_initable_new (G_PASTE_TYPE_##TYPE,                                   \
                                       NULL, /* cancellable */                                \
                                       error,                                                 \
                                       "g-bus-type",       G_BUS_TYPE_SESSION,                \
                                       "g-flags",          G_DBUS_PROXY_FLAGS_NONE,           \
-                                      "g-name",           G_PASTE_##BUS_ID##_BUS_NAME,       \
+                                      "g-name",           BUS_NAME,                          \
                                       "g-object-path",    G_PASTE_##BUS_ID##_OBJECT_PATH,    \
                                       "g-interface-name", G_PASTE_##BUS_ID##_INTERFACE_NAME, \
                                       NULL);                                                 \
@@ -126,7 +77,7 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_PREPARE_EXTRACTION                      \
         GVariantIter result_iter;                    \
         g_variant_iter_init (&result_iter, _result); \
-        G_PASTE_CLEANUP_VARIANT_UNREF GVariant *variant = g_variant_iter_next_value (&result_iter)
+        g_autoptr (GVariant) variant = g_variant_iter_next_value (&result_iter)
 
 #define DBUS_RETURN(if_fail, extract_and_return_answer) \
     if (!_result)                                       \
@@ -165,15 +116,18 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_CALL_TWO_PARAMS_ASYNC_BASE(TYPE_CHECKER, params, method) \
     DBUS_CALL_ASYNC_FULL (TYPE_CHECKER, {}, method, params, 2)
 
+#define DBUS_CALL_THREE_PARAMS_ASYNC_BASE(TYPE_CHECKER, params, method) \
+    DBUS_CALL_ASYNC_FULL (TYPE_CHECKER, {}, method, params, 3)
+
 /**************************************/
 /* Methods / Async / General - Finish */
 /**************************************/
 
-#define DBUS_ASYNC_FINISH_FULL(guard, if_fail, extract_and_return_answer)                            \
-    guard;                                                                                           \
-    G_PASTE_CLEANUP_VARIANT_UNREF GVariant *_result = g_dbus_proxy_call_finish (G_DBUS_PROXY (self), \
-                                                                                result,              \
-                                                                                error);              \
+#define DBUS_ASYNC_FINISH_FULL(guard, if_fail, extract_and_return_answer)         \
+    guard;                                                                        \
+    g_autoptr (GVariant) _result = g_dbus_proxy_call_finish (G_DBUS_PROXY (self), \
+                                                             result,              \
+                                                             error);              \
     DBUS_RETURN (if_fail, extract_and_return_answer)
 
 #define DBUS_ASYNC_FINISH_WITH_RETURN(TYPE_CHECKER, if_fail, extract_and_return_answer)                \
@@ -196,6 +150,9 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_ASYNC_FINISH_RET_BOOL_BASE(TYPE_CHECKER) \
     DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, FALSE, return g_variant_get_boolean (variant))
 
+#define DBUS_ASYNC_FINISH_RET_UINT64_BASE(TYPE_CHECKER) \
+    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, 0, return g_variant_get_uint64 (variant))
+
 #define DBUS_ASYNC_FINISH_RET_UINT32_BASE(TYPE_CHECKER) \
     DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, 0, return g_variant_get_uint32 (variant))
 
@@ -205,26 +162,26 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_ASYNC_FINISH_RET_STRV_BASE(TYPE_CHECKER) \
     DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_variant_dup_strv (variant, NULL))
 
-#define DBUS_ASYNC_FINISH_RET_AU_BASE(TYPE_CHECKER) \
-    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_paste_dbus_get_au_result (variant))
+#define DBUS_ASYNC_FINISH_RET_AT_BASE(TYPE_CHECKER, len) \
+    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_paste_util_get_dbus_at_result (variant, len))
 
-#define DBUS_ASYNC_FINISH_RET_BS_BASE(TYPE_CHECKER) \
-    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, FALSE, GPasteDBusBSResult bs = g_paste_dbus_get_bs_result (variant))
+#define DBUS_ASYNC_FINISH_RET_AU_BASE(TYPE_CHECKER, len) \
+    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_paste_util_get_dbus_au_result (variant, len))
 
 /****************************/
 /* Methods / Sync / General */
 /****************************/
 
-#define DBUS_CALL_FULL(guard, decl, method, params, n_params, if_fail, extract_and_return_answer)                     \
-    guard;                                                                                                            \
-    decl;                                                                                                             \
-    G_PASTE_CLEANUP_VARIANT_UNREF GVariant *_result = g_dbus_proxy_call_sync (G_DBUS_PROXY (self),                    \
-                                                                              method,                                 \
-                                                                              g_variant_new_tuple (params, n_params), \
-                                                                              G_DBUS_CALL_FLAGS_NONE,                 \
-                                                                              -1,                                     \
-                                                                              NULL, /* cancellable */                 \
-                                                                              error);                                 \
+#define DBUS_CALL_FULL(guard, decl, method, params, n_params, if_fail, extract_and_return_answer)  \
+    guard;                                                                                         \
+    decl;                                                                                          \
+    g_autoptr (GVariant) _result = g_dbus_proxy_call_sync (G_DBUS_PROXY (self),                    \
+                                                           method,                                 \
+                                                           g_variant_new_tuple (params, n_params), \
+                                                           G_DBUS_CALL_FLAGS_NONE,                 \
+                                                           -1,                                     \
+                                                           NULL, /* cancellable */                 \
+                                                           error);                                 \
     DBUS_RETURN (if_fail, extract_and_return_answer)
 
 /****************************************/
@@ -265,6 +222,9 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_CALL_TWO_PARAMS_NO_RETURN_BASE(TYPE_CHECKER, params, method) \
     DBUS_CALL_NO_RETURN_BASE (TYPE_CHECKER, {}, method, params, 2)
 
+#define DBUS_CALL_THREE_PARAMS_NO_RETURN_BASE(TYPE_CHECKER, params, method) \
+    DBUS_CALL_NO_RETURN_BASE (TYPE_CHECKER, {}, method, params, 3)
+
 /**************************************************/
 /* Methods / Sync / Impl - With return - No param */
 /**************************************************/
@@ -272,14 +232,20 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, if_fail, variant_extract) \
     DBUS_CALL_WITH_RETURN_BASE (TYPE_CHECKER, {}, method, NULL, 0, if_fail, variant_extract)
 
+#define DBUS_CALL_NO_PARAM_RET_STRING_BASE(TYPE_CHECKER, method) \
+    DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, NULL, return g_variant_dup_string (variant, NULL)) \
+
 #define DBUS_CALL_NO_PARAM_RET_STRV_BASE(TYPE_CHECKER, method) \
     DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, NULL, return g_variant_dup_strv (variant, NULL)) \
 
-#define DBUS_CALL_NO_PARAM_RET_UINT32_BASE(TYPE_CHECKER, method) \
-    DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, 0, return g_variant_get_uint32 (variant)) \
+#define DBUS_CALL_ONE_PARAMV_RET_AT_BASE(TYPE_CHECKER, method, paramv, len) \
+    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_paste_util_get_dbus_at_result (variant, len))
 
-#define DBUS_CALL_ONE_PARAMV_RET_AU_BASE(TYPE_CHECKER, method, paramv) \
-    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_paste_dbus_get_au_result (variant))
+#define DBUS_CALL_ONE_PARAMV_RET_AU_BASE(TYPE_CHECKER, method, paramv, len) \
+    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_paste_util_get_dbus_au_result (variant, len))
+
+#define DBUS_CALL_ONE_PARAMV_RET_STRV_BASE(TYPE_CHECKER, method, paramv) \
+    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_variant_dup_strv (variant, NULL))
 
 /******************************************************/
 /* Methods / Sync / General - With return - One param */
@@ -298,11 +264,14 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_CALL_ONE_PARAM_RET_BOOL_BASE(TYPE_CHECKER, param_type, param_name, method) \
     DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, FALSE, return g_variant_get_boolean (variant))
 
+#define DBUS_CALL_ONE_PARAM_RET_UINT64_BASE(TYPE_CHECKER, param_type, param_name, method) \
+    DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, 0, return g_variant_get_uint64 (variant))
+
 #define DBUS_CALL_ONE_PARAM_RET_STRING_BASE(TYPE_CHECKER, param_type, param_name, method) \
     DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, NULL, return g_variant_dup_string (variant, NULL /* length */))
 
-#define DBUS_CALL_ONE_PARAM_RET_BS_BASE(TYPE_CHECKER, param_type, param_name, method) \
-    DBUS_CALL_ONE_PARAM_RAW_BASE(TYPE_CHECKER, param_type, param_name, method, FALSE, GPasteDBusBSResult bs = g_paste_dbus_get_bs_result (variant))
+#define DBUS_CALL_ONE_PARAM_RET_AT_BASE(TYPE_CHECKER, param_type, param_name, method, len) \
+    DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, NULL, return g_paste_util_get_dbus_at_result (variant, len))
 
 /****************************************************/
 /* Methods / Sync / Impl - With return - Two params */
@@ -311,6 +280,9 @@ g_paste_dbus_get_au_result (GVariant *variant)
 #define DBUS_CALL_TWO_PARAMS_BASE(TYPE_CHECKER, params, method, if_fail, variant_extract) \
     DBUS_CALL_WITH_RETURN_BASE (TYPE_CHECKER, {}, method, params, 2, if_fail, variant_extract)
 
+#define DBUS_CALL_TWO_PARAMS_RET_UINT64_BASE(TYPE_CHECKER, params, method) \
+    DBUS_CALL_TWO_PARAMS_BASE(TYPE_CHECKER, params, method, 0, return g_variant_get_uint64 (variant))
+
 #define DBUS_CALL_TWO_PARAMS_RET_UINT32_BASE(TYPE_CHECKER, params, method) \
     DBUS_CALL_TWO_PARAMS_BASE(TYPE_CHECKER, params, method, 0, return g_variant_get_uint32 (variant))
 
@@ -318,11 +290,11 @@ g_paste_dbus_get_au_result (GVariant *variant)
 /* Properties / Getters */
 /************************/
 
-#define DBUS_GET_PROPERTY_INIT(TYPE_CHECKER, property,  _default)                                           \
-    g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), _default);                                      \
-    G_PASTE_CLEANUP_VARIANT_UNREF GVariant *result = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (self), \
-                                                                                       property);           \
-    if (!result)                                                                                            \
+#define DBUS_GET_PROPERTY_INIT(TYPE_CHECKER, property,  _default)                        \
+    g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), _default);                   \
+    g_autoptr (GVariant) result = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (self), \
+                                                                    property);           \
+    if (!result)                                                                         \
         return _default
 
 #define DBUS_GET_BOOLEAN_PROPERTY_BASE(TYPE_CHECKER, property) \
@@ -337,20 +309,20 @@ g_paste_dbus_get_au_result (GVariant *variant)
 /* Properties / Setters */
 /************************/
 
-#define DBUS_SET_GENERIC_PROPERTY_BASE(TYPE_CHECKER, iface, property, value, vtype)                                 \
-    g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), FALSE);                                                 \
-    GVariant *prop[] = {                                                                                            \
-        g_variant_new_string (iface),                                                                               \
-        g_variant_new_string (property),                                                                            \
-        g_variant_new_variant (g_variant_new_##vtype (value))                                                       \
-    };                                                                                                              \
-    G_PASTE_CLEANUP_VARIANT_UNREF GVariant *result = g_dbus_proxy_call_sync (G_DBUS_PROXY (self),                   \
-                                                                             "org.freedesktop.DBus.Properties.Set", \
-                                                                             g_variant_new_tuple (prop, 3),         \
-                                                                             G_DBUS_CALL_FLAGS_NONE,                \
-                                                                             -1,                                    \
-                                                                             NULL, /* cancellable */                \
-                                                                             error);                                \
+#define DBUS_SET_GENERIC_PROPERTY_BASE(TYPE_CHECKER, iface, property, value, vtype)              \
+    g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), FALSE);                              \
+    GVariant *prop[] = {                                                                         \
+        g_variant_new_string (iface),                                                            \
+        g_variant_new_string (property),                                                         \
+        g_variant_new_variant (g_variant_new_##vtype (value))                                    \
+    };                                                                                           \
+    g_autoptr (GVariant) result = g_dbus_proxy_call_sync (G_DBUS_PROXY (self),                   \
+                                                          "org.freedesktop.DBus.Properties.Set", \
+                                                          g_variant_new_tuple (prop, 3),         \
+                                                          G_DBUS_CALL_FLAGS_NONE,                \
+                                                          -1,                                    \
+                                                          NULL, /* cancellable */                \
+                                                          error);                                \
     return !!result
 
 #define DBUS_SET_BOOLEAN_PROPERTY_BASE(TYPE_CHECKER, iface, property, value) \
