@@ -1,7 +1,7 @@
 /*
  *      This file is part of GPaste.
  *
- *      Copyright 2013 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
+ *      Copyright 2013-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  *
  *      GPaste is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,17 +17,17 @@
  *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gpaste-settings-ui-stack-private.h"
-
-#include <gpaste-config.h>
 #include <gpaste-client.h>
+#include <gpaste-gsettings-keys.h>
 #include <gpaste-settings.h>
+#include <gpaste-settings-ui-stack.h>
 
-#include <glib/gi18n.h>
+struct _GPasteSettingsUiStack
+{
+    GtkStack parent_instance;
+};
 
-#include <stdlib.h>
-
-struct _GPasteSettingsUiStackPrivate
+typedef struct
 {
     GPasteClient    *client;
     GPasteSettings  *settings;
@@ -47,20 +47,20 @@ struct _GPasteSettingsUiStackPrivate
     GtkSpinButton   *max_memory_usage_button;
     GtkSpinButton   *max_text_item_size_button;
     GtkSpinButton   *min_text_item_size_button;
-    GtkEntry        *backup_entry;
+    GtkEntry        *launch_ui_entry;
     GtkEntry        *make_password_entry;
     GtkEntry        *pop_entry;
     GtkEntry        *show_history_entry;
     GtkEntry        *sync_clipboard_to_primary_entry;
     GtkEntry        *sync_primary_to_clipboard_entry;
-    GtkComboBoxText *targets;
+    GtkEntry        *upload_entry;
     gchar         ***actions;
 
     GtkSwitch       *extension_enabled_switch;
     GtkSwitch       *track_extension_state_switch;
 
     gulong           settings_signal;
-};
+} GPasteSettingsUiStackPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GPasteSettingsUiStack, g_paste_settings_ui_stack, GTK_TYPE_STACK)
 
@@ -77,7 +77,7 @@ G_DEFINE_TYPE_WITH_PRIVATE (GPasteSettingsUiStack, g_paste_settings_ui_stack, GT
 #define BOOLEAN_CALLBACK(setting) SETTING_CALLBACK (setting, gboolean)
 #define STRING_CALLBACK(setting)  SETTING_CALLBACK (setting, const gchar *)
 
-#define UINT_CALLBACK(setting) SETTING_CALLBACK_FULL (setting, gdouble, uint)
+#define UINT64_CALLBACK(setting) SETTING_CALLBACK_FULL (setting, gdouble, uint64)
 
 /**
  * g_paste_settings_ui_stack_add_panel:
@@ -111,7 +111,7 @@ BOOLEAN_CALLBACK (synchronize_clipboards)
 BOOLEAN_CALLBACK (track_changes)
 BOOLEAN_CALLBACK (trim_items)
 
-#if G_PASTE_CONFIG_ENABLE_EXTENSION
+#ifdef ENABLE_EXTENSION
 BOOLEAN_CALLBACK (extension_enabled)
 BOOLEAN_CALLBACK (track_extension_state)
 #endif
@@ -128,7 +128,7 @@ g_paste_settings_ui_stack_private_make_behaviour_panel (GPasteSettingsUiStackPri
                                                                                 track_changes_callback,
                                                                                 (GPasteResetCallback) g_paste_settings_reset_track_changes,
                                                                                 settings);
-#if G_PASTE_CONFIG_ENABLE_EXTENSION
+#ifdef ENABLE_EXTENSION
     priv->extension_enabled_switch = g_paste_settings_ui_panel_add_boolean_setting (panel,
                                                                                     _("Enable the gnome-shell extension"),
                                                                                     g_paste_settings_get_extension_enabled (settings),
@@ -185,12 +185,12 @@ g_paste_settings_ui_stack_private_make_behaviour_panel (GPasteSettingsUiStackPri
     return panel;
 }
 
-UINT_CALLBACK (element_size)
-UINT_CALLBACK (max_displayed_history_size)
-UINT_CALLBACK (max_history_size)
-UINT_CALLBACK (max_memory_usage)
-UINT_CALLBACK (max_text_item_size)
-UINT_CALLBACK (min_text_item_size)
+UINT64_CALLBACK (element_size)
+UINT64_CALLBACK (max_displayed_history_size)
+UINT64_CALLBACK (max_history_size)
+UINT64_CALLBACK (max_memory_usage)
+UINT64_CALLBACK (max_text_item_size)
+UINT64_CALLBACK (min_text_item_size)
 
 static GPasteSettingsUiPanel *
 g_paste_settings_ui_stack_private_make_history_settings_panel (GPasteSettingsUiStackPrivate *priv)
@@ -201,42 +201,42 @@ g_paste_settings_ui_stack_private_make_history_settings_panel (GPasteSettingsUiS
     priv->element_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
                                                                              _("Max element size when displaying: "),
                                                                              (gdouble) g_paste_settings_get_element_size (settings),
-                                                                             0, 255, 5,
+                                                                             0, 511, 5,
                                                                              element_size_callback,
                                                                              (GPasteResetCallback) g_paste_settings_reset_element_size,
                                                                              settings);
     priv->max_displayed_history_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
                                                                                            _("Max displayed history size: "),
                                                                                            (gdouble) g_paste_settings_get_max_displayed_history_size (settings),
-                                                                                           5, 255, 5,
+                                                                                           10, 255, 5,
                                                                                            max_displayed_history_size_callback,
                                                                                            (GPasteResetCallback) g_paste_settings_reset_max_displayed_history_size,
                                                                                            settings);
     priv->max_history_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
                                                                                  _("Max history size: "),
                                                                                  (gdouble) g_paste_settings_get_max_history_size (settings),
-                                                                                 5, 255, 5,
+                                                                                 100, 65535, 5,
                                                                                  max_history_size_callback,
                                                                                  (GPasteResetCallback) g_paste_settings_reset_max_history_size,
                                                                                  settings);
     priv->max_memory_usage_button = g_paste_settings_ui_panel_add_range_setting (panel,
                                                                                  _("Max memory usage (MB): "),
                                                                                  (gdouble) g_paste_settings_get_max_memory_usage (settings),
-                                                                                 5, 2000, 5,
+                                                                                 5, 16383, 5,
                                                                                  max_memory_usage_callback,
                                                                                  (GPasteResetCallback) g_paste_settings_reset_max_memory_usage,
                                                                                  settings);
     priv->max_text_item_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
                                                                                    _("Max text item length: "),
                                                                                    (gdouble) g_paste_settings_get_max_text_item_size (settings),
-                                                                                   1, G_MAXUINT, 1,
+                                                                                   1, G_MAXUINT64, 1,
                                                                                    max_text_item_size_callback,
                                                                                    (GPasteResetCallback) g_paste_settings_reset_max_text_item_size,
                                                                                    settings);
     priv->min_text_item_size_button = g_paste_settings_ui_panel_add_range_setting (panel,
                                                                                    _("Min text item length: "),
                                                                                    (gdouble) g_paste_settings_get_min_text_item_size (settings),
-                                                                                   1, G_MAXUINT, 1,
+                                                                                   1, 65535, 1,
                                                                                    min_text_item_size_callback,
                                                                                    (GPasteResetCallback) g_paste_settings_reset_min_text_item_size,
                                                                                    settings);
@@ -244,11 +244,13 @@ g_paste_settings_ui_stack_private_make_history_settings_panel (GPasteSettingsUiS
     return panel;
 }
 
+STRING_CALLBACK (launch_ui)
 STRING_CALLBACK (make_password)
 STRING_CALLBACK (pop)
 STRING_CALLBACK (show_history)
 STRING_CALLBACK (sync_clipboard_to_primary)
 STRING_CALLBACK (sync_primary_to_clipboard)
+STRING_CALLBACK (upload)
 
 static GPasteSettingsUiPanel *
 g_paste_settings_ui_stack_private_make_keybindings_panel (GPasteSettingsUiStackPrivate *priv)
@@ -263,6 +265,13 @@ g_paste_settings_ui_stack_private_make_keybindings_panel (GPasteSettingsUiStackP
                                                                   pop_callback,
                                                                   (GPasteResetCallback) g_paste_settings_reset_pop,
                                                                   settings);
+    /* translators: Keyboard shortcut to launch the graphical tool */
+    priv->launch_ui_entry = g_paste_settings_ui_panel_add_text_setting (panel,
+                                                                        _("Launch the graphical tool: "),
+                                                                        g_paste_settings_get_launch_ui (settings),
+                                                                        launch_ui_callback,
+                                                                        (GPasteResetCallback) g_paste_settings_reset_launch_ui,
+                                                                        settings);
     /* translators: Keyboard shortcut to mark the active item as being a password */
     priv->make_password_entry = g_paste_settings_ui_panel_add_text_setting (panel,
                                                                             _("Mark the active item as being a password: "),
@@ -291,6 +300,13 @@ g_paste_settings_ui_stack_private_make_keybindings_panel (GPasteSettingsUiStackP
                                                                                         sync_primary_to_clipboard_callback,
                                                                                         (GPasteResetCallback) g_paste_settings_reset_sync_primary_to_clipboard,
                                                                                         settings);
+    /* translators: Keyboard shortcut to upload the active item from history to a pastebin service */
+    priv->upload_entry = g_paste_settings_ui_panel_add_text_setting (panel,
+                                                                     _("Upload the active item to a pastebin service: "),
+                                                                     g_paste_settings_get_upload (settings),
+                                                                     upload_callback,
+                                                                     (GPasteResetCallback) g_paste_settings_reset_upload,
+                                                                     settings);
 
     return panel;
 }
@@ -305,98 +321,6 @@ g_paste_settings_ui_check_connection_error (GError *error)
     return TRUE;
 }
 
-static void
-g_paste_settings_ui_stack_private_refill_histories (GPasteSettingsUiStackPrivate *priv)
-{
-    G_PASTE_CLEANUP_ERROR_FREE GError *error = NULL;
-    GStrv histories = g_paste_client_list_histories_sync (priv->client, &error);
-
-    if (g_paste_settings_ui_check_connection_error (error))
-        return;
-
-    GtkComboBoxText *targets = priv->targets;
-
-    gtk_combo_box_text_remove_all (targets);
-
-    for (guint i = 0; histories[i]; ++i)
-        gtk_combo_box_text_append (targets, histories[i], histories[i]);
-
-    gtk_combo_box_set_active_id (GTK_COMBO_BOX (targets),
-                                 g_paste_settings_get_history_name (priv->settings));
-}
-
-static void
-dummy_callback (const gchar *value G_GNUC_UNUSED,
-                gpointer     user_data G_GNUC_UNUSED)
-{
-}
-
-static void
-backup_callback (const gchar *value,
-                 gpointer     user_data)
-{
-    GPasteSettingsUiStackPrivate *priv = user_data;
-    G_PASTE_CLEANUP_ERROR_FREE GError *error = NULL;
-
-    g_paste_client_backup_history_sync (priv->client, value, &error);
-
-    if (g_paste_settings_ui_check_connection_error (error))
-        return;
-
-    g_paste_settings_ui_stack_private_refill_histories (priv);
-}
-
-static void
-targets_callback (const gchar *action,
-                  const gchar *target,
-                  gpointer     user_data)
-{
-    GPasteSettingsUiStackPrivate *priv = user_data;
-    GPasteClient *client = priv->client;
-    G_PASTE_CLEANUP_ERROR_FREE GError *error = NULL;
-
-    if (!g_strcmp0 (action, "switch"))
-        g_paste_client_switch_history_sync (client, target, &error);
-    else if (!g_strcmp0 (action, "delete"))
-        g_paste_client_delete_history_sync (client, target, &error);
-    else
-        fprintf (stderr, "unknown action: %s\n", action);
-
-    if (g_paste_settings_ui_check_connection_error (error))
-        return;
-
-    g_paste_settings_ui_stack_private_refill_histories (priv);
-}
-
-static GPasteSettingsUiPanel *
-g_paste_settings_ui_stack_private_make_histories_panel (GPasteSettingsUiStackPrivate *priv)
-{
-    GPasteSettings *settings = priv->settings;
-    GPasteSettingsUiPanel *panel = g_paste_settings_ui_panel_new ();
-
-    G_PASTE_CLEANUP_FREE gchar *backup_name = g_strconcat (g_paste_settings_get_history_name (settings), "_backup", NULL);
-    priv->backup_entry = g_paste_settings_ui_panel_add_text_confirm_setting (panel,
-                                                                             _("Backup history as: "),
-                                                                             backup_name,
-                                                                             dummy_callback,
-                                                                             NULL,
-                                                                             /* translators: This is the name of a multi-history management action */
-                                                                             _("Backup"),
-                                                                             backup_callback,
-                                                                             priv);
-
-    /* translators: This is the text displayed on the button used to perform a multi-history management action */
-    priv->targets = g_paste_settings_ui_panel_add_multi_action_setting (panel,
-                                                                        (GStrv const *) priv->actions,
-                                                                        _("Ok"),
-                                                                        targets_callback,
-                                                                        priv);
-
-    g_paste_settings_ui_stack_private_refill_histories (priv);
-
-    return panel;
-}
-
 /**
  * g_paste_settings_ui_stack_fill:
  * @self: a #GPasteSettingsUiStack instance
@@ -413,7 +337,6 @@ g_paste_settings_ui_stack_fill (GPasteSettingsUiStack *self)
     g_paste_settings_ui_stack_add_panel (self, "general",   _("General behaviour"),  g_paste_settings_ui_stack_private_make_behaviour_panel (priv));
     g_paste_settings_ui_stack_add_panel (self, "history",   _("History settings"),   g_paste_settings_ui_stack_private_make_history_settings_panel (priv));
     g_paste_settings_ui_stack_add_panel (self, "keyboard",  _("Keyboard shortcuts"), g_paste_settings_ui_stack_private_make_keybindings_panel (priv));
-    g_paste_settings_ui_stack_add_panel (self, "histories", _("Histories"),          g_paste_settings_ui_stack_private_make_histories_panel (priv));
 }
 
 static void
@@ -427,14 +350,10 @@ g_paste_settings_ui_stack_settings_changed (GPasteSettings *settings,
         gtk_spin_button_set_value (priv->element_size_button, g_paste_settings_get_element_size (settings));
     else if (!g_strcmp0 (key, G_PASTE_GROWING_LINES_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->growing_lines_switch), g_paste_settings_get_growing_lines (settings));
-    else if (!g_strcmp0 (key, G_PASTE_HISTORY_NAME_SETTING))
-    {
-        G_PASTE_CLEANUP_FREE gchar *text = g_strconcat (g_paste_settings_get_history_name (settings), "_backup", NULL);
-        gtk_entry_set_text (priv->backup_entry, text);
-        g_paste_settings_ui_stack_private_refill_histories (priv);
-    }
     else if (!g_strcmp0 (key, G_PASTE_IMAGES_SUPPORT_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->images_support_switch), g_paste_settings_get_images_support (settings));
+    else if (!g_strcmp0 (key, G_PASTE_LAUNCH_UI_SETTING))
+        gtk_entry_set_text (priv->launch_ui_entry, g_paste_settings_get_launch_ui (settings));
     else if (!g_strcmp0 (key, G_PASTE_MAKE_PASSWORD_SETTING))
         gtk_entry_set_text (priv->make_password_entry, g_paste_settings_get_make_password (settings));
     else if (!g_strcmp0 (key, G_PASTE_MAX_DISPLAYED_HISTORY_SIZE_SETTING))
@@ -459,13 +378,15 @@ g_paste_settings_ui_stack_settings_changed (GPasteSettings *settings,
         gtk_entry_set_text (priv->sync_clipboard_to_primary_entry, g_paste_settings_get_sync_clipboard_to_primary (settings));
     else if (!g_strcmp0 (key, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING))
         gtk_entry_set_text (priv->sync_primary_to_clipboard_entry, g_paste_settings_get_sync_primary_to_clipboard (settings));
+    else if (!g_strcmp0 (key, G_PASTE_UPLOAD_SETTING))
+        gtk_entry_set_text (priv->upload_entry, g_paste_settings_get_upload (settings));
     else if (!g_strcmp0 (key, G_PASTE_SYNCHRONIZE_CLIPBOARDS_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->synchronize_clipboards_switch), g_paste_settings_get_synchronize_clipboards (settings));
     else if (!g_strcmp0 (key, G_PASTE_TRACK_CHANGES_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->track_changes_switch), g_paste_settings_get_track_changes (settings));
     else if (!g_strcmp0 (key, G_PASTE_TRIM_ITEMS_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->trim_items_switch), g_paste_settings_get_trim_items (settings));
-#if G_PASTE_CONFIG_ENABLE_EXTENSION
+#ifdef ENABLE_EXTENSION
     else if (!g_strcmp0 (key, G_PASTE_EXTENSION_ENABLED_SETTING))
         gtk_switch_set_active (GTK_SWITCH (priv->extension_enabled_switch), g_paste_settings_get_extension_enabled (settings));
     else if (!g_strcmp0 (key, G_PASTE_TRACK_EXTENSION_STATE_SETTING))
@@ -494,7 +415,7 @@ g_paste_settings_ui_stack_finalize (GObject *object)
     GPasteSettingsUiStackPrivate *priv = g_paste_settings_ui_stack_get_instance_private (G_PASTE_SETTINGS_UI_STACK (object));
     GStrv *actions = priv->actions;
 
-    for (guint i = 0; actions[i]; ++i)
+    for (guint64 i = 0; actions[i]; ++i)
         g_free ((GStrv) actions[i]);
     g_free ((GStrv *) actions);
 
@@ -544,7 +465,7 @@ g_paste_settings_ui_stack_init (GPasteSettingsUiStack *self)
  *
  * Create a new instance of #GPasteSettingsUiStack
  *
- * Returns: (nullable): a newly allocated #GPasteSettingsUiStack
+ * Returns: (optional): a newly allocated #GPasteSettingsUiStack
  *                      free it with g_object_unref
  */
 G_PASTE_VISIBLE GPasteSettingsUiStack *
